<?php

namespace Bixo\Listings\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Listings\Http\Requests\ListingRequest;
use Bixo\Listings\Interfaces\ListingRepositoryInterface;
use Bixo\Listings\Models\Listing;

/**
 * Resource controller class for listing.
 */
class ListingResourceController extends BaseController
{

    /**
     * Initialize listing resource controller.
     *
     * @param type ListingRepositoryInterface $listing
     *
     * @return null
     */
    public function __construct(ListingRepositoryInterface $listing)
    {
        parent::__construct();
        $this->repository = $listing;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Listings\Repositories\Criteria\ListingResourceCriteria::class);
    }

    /**
     * Display a list of listing.
     *
     * @return Response
     */
    public function index(ListingRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Listings\Repositories\Presenter\ListingPresenter::class)
                ->$function();
        }

        $listings = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('listings::listing.names'))
            ->view('listings::listing.index', true)
            ->data(compact('listings', 'view'))
            ->output();
    }

    /**
     * Display listing.
     *
     * @param Request $request
     * @param Model   $listing
     *
     * @return Response
     */
    public function show(ListingRequest $request, Listing $listing)
    {

        if ($listing->exists) {
            $view = 'listings::listing.show';
        } else {
            $view = 'listings::listing.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('listings::listing.name'))
            ->data(compact('listing'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new listing.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ListingRequest $request)
    {

        $listing = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('listings::listing.name')) 
            ->view('listings::listing.create', true) 
            ->data(compact('listing'))
            ->output();
    }

    /**
     * Create new listing.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ListingRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $listing                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('listings::listing.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('listings/listing/' . $listing->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/listings/listing'))
                ->redirect();
        }

    }

    /**
     * Show listing for editing.
     *
     * @param Request $request
     * @param Model   $listing
     *
     * @return Response
     */
    public function edit(ListingRequest $request, Listing $listing)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('listings::listing.name'))
            ->view('listings::listing.edit', true)
            ->data(compact('listing'))
            ->output();
    }

    /**
     * Update the listing.
     *
     * @param Request $request
     * @param Model   $listing
     *
     * @return Response
     */
    public function update(ListingRequest $request, Listing $listing)
    {
        try {
            $attributes = $request->all();

            $listing->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('listings::listing.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('listings/listing/' . $listing->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('listings/listing/' . $listing->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the listing.
     *
     * @param Model   $listing
     *
     * @return Response
     */
    public function destroy(ListingRequest $request, Listing $listing)
    {
        try {

            $listing->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('listings::listing.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('listings/listing/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('listings/listing/' . $listing->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple listing.
     *
     * @param Model   $listing
     *
     * @return Response
     */
    public function delete(ListingRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('listings::listing.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('listings/listing'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/listings/listing'))
                ->redirect();
        }

    }

    /**
     * Restore deleted listings.
     *
     * @param Model   $listing
     *
     * @return Response
     */
    public function restore(ListingRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('listings::listing.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/listings/listing'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/listings/listing/'))
                ->redirect();
        }

    }

}
